package io.tswf.transmutations.staticview

import groovy.transform.CompileStatic
import io.tswf.transmutations.common.annotation.ClassReference
import io.tswf.transmutations.common.annotation.MethodDescription
import io.tswf.transmutations.staticview.annotation.StaticMethodInvoke
import io.tswf.transmutations.staticview.annotation.StaticView
import io.tswf.transmutations.staticview.generation.StaticViewInjector
import io.tswf.transmutations.staticview.reading.StaticViewMetadataReader

@CompileStatic
class StaticViewApp {
    static void main(String[] args) {
        def staticViews = TestStaticViews.toClassNode()
        def viewMeta = StaticViewMetadataReader.readStaticViewMetadata(staticViews)
        def targetClass = ClassLoader.contextClassloader().loadClass(viewMeta.targetType.className).toClassNode()

        StaticViewInjector.injectStaticView(viewMeta, targetClass)

        new File("static-views-owner.class").bytes = staticViews.toByteArray()
        new File("static-views-target.class").bytes = targetClass.toByteArray()
    }
}

@StaticView(targetClass = @ClassReference(TestStaticViewsTarget))
class TestStaticViews {
    @StaticMethodInvoke(
            @MethodDescription(name = "helloWorld", parameterTypeClasses = [ String, String ], returnTypeClass = String)
    )
    static Object callHelloWorld(Object s1, int s2) {  }
    static TestStaticViewsTarget newInstance(String a) { null }
    static String getSuperSecretString() { null }
    static void setSuperSecretString(String newValue) {}
}

class TestStaticViewsTarget {

    private static final String superSecretString = ""

    private TestStaticViewsTarget(String s) {}

    private static String helloWorld(String s1, String s2) {  }
}