package io.tswf.transmutations.staticview.exception

import groovy.transform.CompileStatic
import io.tswf.transmutations.staticview.meta.StaticViewMethodMetadata
import io.tswf.transmutations.staticview.reading.IStaticViewMethodMapperReader

@CompileStatic
class MethodMapperReadingException extends RuntimeException {
    StaticViewMethodMetadata viewMethodMeta
    IStaticViewMethodMapperReader mapperReader

    MethodMapperReadingException(StaticViewMethodMetadata viewMethodMeta, IStaticViewMethodMapperReader mapperReader) {
        this.viewMethodMeta = viewMethodMeta
        this.mapperReader = mapperReader
    }

    MethodMapperReadingException(String message, StaticViewMethodMetadata viewMethodMeta, IStaticViewMethodMapperReader mapperReader) {
        super(getExceptionMessage(message, viewMethodMeta, mapperReader))
        this.viewMethodMeta = viewMethodMeta
        this.mapperReader = mapperReader
    }

    MethodMapperReadingException(String message, Throwable cause, StaticViewMethodMetadata viewMethodMeta, IStaticViewMethodMapperReader mapperReader) {
        super(getExceptionMessage(message, viewMethodMeta, mapperReader), cause)
        this.viewMethodMeta = viewMethodMeta
        this.mapperReader = mapperReader
    }

    MethodMapperReadingException(Throwable cause, StaticViewMethodMetadata viewMethodMeta, IStaticViewMethodMapperReader mapperReader) {
        super(getExceptionMessage("", viewMethodMeta, mapperReader), cause)
        this.viewMethodMeta = viewMethodMeta
        this.mapperReader = mapperReader
    }

    static String getExceptionMessage(String message, StaticViewMethodMetadata viewMethodMeta, IStaticViewMethodMapperReader mapperReader) {
        return "Exception occurred while trying to read static view method mapper ${viewMethodMeta.viewMethod.javaFullName} from ${viewMethodMeta.view.viewClass.javaName} by ${mapperReader}: ${message}"
    }
}
