package io.tswf.transmutations.staticview.generation

import groovy.transform.CompileStatic
import groovy.transform.NamedVariant
import io.tswf.transmutations.staticview.meta.StaticViewMethodMetadata
import io.tswf.groovy.bettergroovy.transforms.directnamedargs.DirectNamedArgsCall
import javafx.beans.NamedArg
import org.objectweb.asm.tree.MethodNode

import static org.objectweb.asm.Opcodes.*

@CompileStatic
class StaticAccessMethodGenerator {

    public static final String VIEW_CLASS_ACTOR = '${viewClassSimpleName}'
    public static final String VIEW_METHOD_NAME_ACTOR = '${viewMethodName}'
    public static final String ACCESSOR_METHOD_TEMPLATE = "__access\$${VIEW_CLASS_ACTOR}_${VIEW_METHOD_NAME_ACTOR}"


    static MethodNode generateStaticAccessor(StaticViewMethodMetadata viewMethodMeta) {
        def viewMethodName = viewMethodMeta.viewMethod.name
        def viewClassSimpleName = viewMethodMeta.view.viewClass.javaSimpleName

        def accessorName = formatAccessorName(viewClassSimpleName: viewClassSimpleName, viewMethodName: viewMethodName)

        return new MethodNode(
                ACC_PUBLIC | ACC_STATIC | ACC_SYNTHETIC,
                accessorName,
                viewMethodMeta.viewMethod.desc,
                null,
                null
        )
    }

    @NamedVariant
    static String formatAccessorName(String viewClassSimpleName, String viewMethodName) {
        return ACCESSOR_METHOD_TEMPLATE
            .replace(VIEW_CLASS_ACTOR, viewClassSimpleName)
            .replace(VIEW_METHOD_NAME_ACTOR, viewMethodName)
    }
}
