package io.tswf.transmutations.staticview.generation.enricher

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.staticview.generation.IStaticViewClassEnricher
import io.tswf.transmutations.staticview.generation.StaticAccessMethodGenerator
import io.tswf.transmutations.staticview.meta.StaticViewMetadata
import org.objectweb.asm.tree.InsnNode
import org.objectweb.asm.tree.MethodInsnNode
import org.objectweb.asm.tree.VarInsnNode

import java.lang.reflect.Modifier

import static org.objectweb.asm.Opcodes.*

@AutoService(IStaticViewClassEnricher)
@CompileStatic
class FactoryMethodStaticClassEnricher implements IStaticViewClassEnricher {

    @Override
    void enrichClass(StaticViewMetadata viewMeta) {
        for (def viewMethodMeta : viewMeta.viewMethodsMetadata) {
            def accessorMethod = StaticAccessMethodGenerator.generateStaticAccessor(viewMethodMeta)
            def viewMethod = viewMethodMeta.viewMethod

            def viewMethodParameterTypes = viewMethod.parameterTypes
            def isViewMethodStatic = Modifier.isStatic(viewMethod.access)

            viewMethod.instructions.clear()

            if (!isViewMethodStatic) {
                viewMethod.instructions << new VarInsnNode(ALOAD, 0)
            }

            viewMethodParameterTypes.eachWithIndex { parameterType, parameterIndex ->
                def localVariableToLoadIndex = isViewMethodStatic ? parameterIndex : parameterIndex + 1
                viewMethod.instructions << new VarInsnNode(parameterType.getOpcode(ILOAD), localVariableToLoadIndex)
            }

            viewMethod.instructions << new MethodInsnNode(INVOKESTATIC, viewMeta.targetType.internalName, accessorMethod.name, accessorMethod.desc, false)
            viewMethod.instructions << new InsnNode(accessorMethod.returnType.getOpcode(IRETURN))
        }
    }
}
