package io.tswf.transmutations.staticview.generation.mapper


import groovy.transform.CompileStatic
import io.tswf.transmutations.staticview.exception.InvalidStaticViewMethodException
import io.tswf.transmutations.staticview.generation.IStaticViewMethodMapper
import io.tswf.transmutations.staticview.generation.StaticViewMethodMappingContext
import io.tswf.transmutations.staticview.util.TypeMatchingUtil
import org.objectweb.asm.Type
import org.objectweb.asm.tree.FieldInsnNode
import org.objectweb.asm.tree.InsnNode

import java.lang.reflect.Modifier

import static org.objectweb.asm.Opcodes.*

@CompileStatic
class FieldGetStaticViewMethodMapper implements IStaticViewMethodMapper {

    String fieldName
    Type fieldType

    FieldGetStaticViewMethodMapper(String fieldName, Type fieldType) {
        this.fieldName = fieldName
        this.fieldType = fieldType
    }

    @Override
    void handleStaticViewMethodMapping(StaticViewMethodMappingContext context) {
        def fieldToAccess = context.targetClass.getFieldByNameAndType(fieldName, fieldType)

        def accessor = context.generatedAccessMethodInTarget
        def accessorReturnType = accessor.returnType

        if (TypeMatchingUtil.areTypesMismatching(accessorReturnType, fieldType)) {
            throw new InvalidStaticViewMethodException("accessor method return type ${accessorReturnType.className} is not matches with field type ${fieldType.className}", context.viewMethodMeta)
        }

        def isFieldToAccessStatic = Modifier.isStatic(fieldToAccess.access)

        if (!isFieldToAccessStatic) {
            throw new InvalidStaticViewMethodException("target field '${fieldName}: ${fieldType.className}' is not static", context.viewMethodMeta)
        }

        accessor.instructions << new FieldInsnNode(GETSTATIC, context.targetClass.name, fieldName, fieldType.descriptor)
        accessor.instructions << new InsnNode(accessorReturnType.getOpcode(IRETURN))
    }
}
