package io.tswf.transmutations.staticview.generation.mapper

import groovy.transform.CompileStatic
import io.tswf.transmutations.staticview.exception.InvalidStaticViewMethodException
import io.tswf.transmutations.staticview.generation.StaticViewMethodMappingContext
import io.tswf.transmutations.staticview.util.TypeMatchingUtil
import org.objectweb.asm.commons.Method
import org.objectweb.asm.tree.InsnNode
import org.objectweb.asm.tree.MethodInsnNode
import org.objectweb.asm.tree.MethodNode

import static org.objectweb.asm.Opcodes.INVOKESTATIC
import static org.objectweb.asm.Opcodes.IRETURN

@CompileStatic
class MethodInvokeStaticViewMethodMapper extends AbstractMethodInvokeStaticViewMethodMapper {

    MethodInvokeStaticViewMethodMapper(Method targetMethodDescription) {
        super(targetMethodDescription)
    }

    @Override
    void applyAdditionalValidations(StaticViewMethodMappingContext context, MethodNode accessorMethod, MethodNode targetMethod) {
        // Сравниваем для тех же целей возвращаемые значения методов
        def targetMethodReturnType = targetMethod.returnType
        def accessorMethodReturnType = accessorMethod.returnType

        if (TypeMatchingUtil.areTypesMismatching(accessorMethodReturnType, targetMethodReturnType)) {
            throw new InvalidStaticViewMethodException("target method ${targetMethod.javaFullName} return type ${targetMethodReturnType.className} is missmatching with ${accessorMethodReturnType.className} return type of accessor method ${accessorMethod.javaFullName}", context.viewMethodMeta)
        }
    }

    @Override
    void appendMethodInvocation(StaticViewMethodMappingContext context, MethodNode accessorMethod, MethodNode targetMethod) {

        // Загружаем параметры для вызова
        loadInvocationArguments(context, accessorMethod, targetMethod)

        // Добавляем вызов целевого метода
        accessorMethod.instructions << new MethodInsnNode(INVOKESTATIC, context.targetClass.name, targetMethodDescription.name, targetMethodDescription.descriptor, false)

        // Передаем возвращенное значени в метод-аксессор
        accessorMethod.instructions << new InsnNode(accessorMethod.returnType.getOpcode(IRETURN))
    }
}
