package io.tswf.transmutations.staticview.reading

import groovy.transform.CompileStatic
import io.tswf.transmutations.staticview.annotation.StaticView
import io.tswf.transmutations.staticview.exception.InvalidStaticViewClassException
import io.tswf.transmutations.staticview.exception.StaticViewClassReadingException
import io.tswf.transmutations.staticview.meta.StaticViewMetadata
import io.tswf.transmutations.staticview.reading.exclude.ViewMethodMetadataExclusions
import io.tswf.transmutations.common.reader.ClassReferenceReader
import io.tswf.transmutations.common.remapping.TransmutationRemapper
import org.objectweb.asm.tree.AnnotationNode
import org.objectweb.asm.tree.ClassNode

import java.lang.reflect.Modifier

@CompileStatic
class StaticViewMetadataReader {

    public static final String TARGET_CLASS_PARAMETER_NAME = "targetClass"

    static StaticViewMetadata readStaticViewMetadata(ClassNode viewClass) {
        try {
            def result = new StaticViewMetadata()
            result.viewClass = viewClass

            def viewAnnotation = viewClass.getAnnotationByType(StaticView)

            def targetType = ClassReferenceReader.getTypeFromClassRefAnnotation(
                    viewAnnotation.getValueByName(TARGET_CLASS_PARAMETER_NAME, AnnotationNode)
            )

            result.targetType = targetType

            def viewMethodCandidates = viewClass.methods
                .stream()
                    .filter { Modifier.isStatic(it.access) }
                    .filter { !ViewMethodMetadataExclusions.isNeedsToExcludeMethod(result, it) }
                    .toList()

            for (def viewMethodCandidate : viewMethodCandidates) {
                def viewMethodMeta = StaticViewMethodMetadataReader.readStaticViewMethodMeta(result, viewMethodCandidate)
                result.viewMethodsMetadata << viewMethodMeta
            }

            result.targetType = TransmutationRemapper.mapType(result.targetType)

            return result
        } catch (Throwable e) {
            if (e instanceof StaticViewClassReadingException.Bypass) {
                throw e
            }

            throw new StaticViewClassReadingException(e, viewClass)
        }
    }

}
