package io.tswf.transmutations.staticview.reading

import groovy.transform.CompileStatic
import io.tswf.transmutations.staticview.exception.MethodMapperReadingException
import io.tswf.transmutations.staticview.exception.MultipleStaticViewMethodMappersFoundException
import io.tswf.transmutations.staticview.exception.NoStaticViewMethodMapperFoundException
import io.tswf.transmutations.staticview.meta.StaticViewMetadata
import io.tswf.transmutations.staticview.meta.StaticViewMethodMetadata
import io.tswf.transmutations.common.conflict.ConflictStrategyReader
import org.objectweb.asm.tree.MethodNode

@CompileStatic
class StaticViewMethodMetadataReader {

    @Lazy
    static List<IStaticViewMethodMapperReader> methodMapperReaders = ServiceLoader.load(IStaticViewMethodMapperReader)
            .toList()


    static StaticViewMethodMetadata readStaticViewMethodMeta(StaticViewMetadata viewMeta, MethodNode methodNode) {
        def result = new StaticViewMethodMetadata(
                viewMethod: methodNode,
                view: viewMeta,
                mapper: null
        )

        result.conflictStrategy = ConflictStrategyReader.readConflictStrategy(methodNode)

        def mapperReadingContext = new StaticViewMethodMapperReadingContext(viewMethodMeta: result)

        for (def methodMapperReader : methodMapperReaders) {
            try {
                methodMapperReader.handleStaticViewMethodReading(mapperReadingContext)
            } catch (Throwable e) {
                throw new MethodMapperReadingException(e, result, methodMapperReader)
            }
        }

        if (mapperReadingContext.result.isNullOrEmpty()) {
            throw new NoStaticViewMethodMapperFoundException(result)
        }

        if (mapperReadingContext.result.size() > 1) {
            throw new MultipleStaticViewMethodMappersFoundException(result, mapperReadingContext.result)
        }

        result.mapper = mapperReadingContext.result.first()

        return result
    }

}
