package io.tswf.transmutations.staticview.reading.mapper

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.common.remapping.TransmutationRemapper
import io.tswf.transmutations.staticview.annotation.StaticFieldAccessor
import io.tswf.transmutations.staticview.generation.mapper.FieldGetStaticViewMethodMapper
import io.tswf.transmutations.staticview.generation.mapper.FieldSetStaticViewMethodMapper
import io.tswf.transmutations.staticview.reading.IStaticViewMethodMapperReader
import io.tswf.transmutations.staticview.reading.StaticViewMethodMapperReadingContext
import io.tswf.transmutations.common.accessors.AccessorHelper
import io.tswf.transmutations.common.reader.FieldDescriptionReader
import org.objectweb.asm.tree.AnnotationNode

@AutoService(IStaticViewMethodMapperReader)
@CompileStatic
class FieldAccessorStaticViewMapperReader implements IStaticViewMethodMapperReader {

    public static final Collection<String> PREFIXES = AccessorHelper.GETTER_PREFIXES + AccessorHelper.SETTER_PREFIXES
    public static final String FIELD_PARAMETER_NAME = "value"

    @Override
    void handleStaticViewMethodReading(StaticViewMethodMapperReadingContext context) {
        def fieldAccessorAnnotation = context.targetMethod.findAnnotationByType(StaticFieldAccessor)
        def isNameStartsFromPrefix = PREFIXES.any { context.targetMethod.name.startsWith(it) }

        def isTargetMethodFieldAccessor = fieldAccessorAnnotation != null || isNameStartsFromPrefix

        if (isTargetMethodFieldAccessor) {

            def rawFieldName, rawFieldType

            if (fieldAccessorAnnotation != null) {
                def fieldDescriptionAnnotation = fieldAccessorAnnotation.findValueByName(FIELD_PARAMETER_NAME, AnnotationNode)

                if (fieldDescriptionAnnotation != null) {
                    (rawFieldName, rawFieldType) = FieldDescriptionReader.findFieldDescriptionFromAnnotation(fieldDescriptionAnnotation)
                }
            }

            def isSetter = context.targetMethod.parameterTypes.isNotNullOrEmpty()

            if (rawFieldName == null || rawFieldType == null) {
                rawFieldName = isSetter
                        ? AccessorHelper.resolveFieldNameFromSetterMethod(context.targetMethod)
                        : AccessorHelper.resolveFieldNameFromGetterMethod(context.targetMethod)

                rawFieldType = AccessorHelper.getTypeFromAccessor(context.targetMethod)
            }

            def result

            def (mappedFieldName, mappedFieldType) = TransmutationRemapper.mapField(context.viewMethodMeta.view.targetType, rawFieldName, rawFieldType)

            if (isSetter) {
                AccessorHelper.validateSetter(context.targetMethod)
                result = new FieldSetStaticViewMethodMapper(mappedFieldName, mappedFieldType)
            } else {
                AccessorHelper.validateGetter(context.targetMethod)
                result = new FieldGetStaticViewMethodMapper(mappedFieldName, mappedFieldType)
            }

            context.result << result
        }
    }
}
