package io.tswf.transmutations.staticview.reading.mapper

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.common.remapping.TransmutationRemapper
import io.tswf.transmutations.staticview.annotation.StaticMethodInvoke
import io.tswf.transmutations.staticview.generation.mapper.MethodInvokeStaticViewMethodMapper
import io.tswf.transmutations.staticview.reading.IStaticViewMethodMapperReader
import io.tswf.transmutations.staticview.reading.StaticViewMethodMapperReadingContext
import io.tswf.transmutations.common.reader.MethodDescriptionReader
import org.objectweb.asm.commons.Method
import org.objectweb.asm.tree.AnnotationNode

@AutoService(IStaticViewMethodMapperReader)
@CompileStatic
class MethodInvokeStaticViewMethodMapperReader implements IStaticViewMethodMapperReader {

    public static final String METHOD_DESCRIPTION_PARAMETER_NAME = "value"
    public static final Collection<String> PREFIXES = [ "call", "invoke", "run" ]

    @Override
    void handleStaticViewMethodReading(StaticViewMethodMapperReadingContext context) {
        def invokeMethodAnnotation = context.targetMethod.findAnnotationByType(StaticMethodInvoke)
        def isMethodNameStartsWithPrefix = PREFIXES.any { context.targetMethod.name.startsWith(it) }

        def isMethodInvoke = invokeMethodAnnotation != null || isMethodNameStartsWithPrefix

        if (isMethodInvoke) {
            def rawMethodToInvokeDescription

            if (invokeMethodAnnotation != null) {
                def methodToInvokeDescriptionAnnotation = invokeMethodAnnotation.findValueByName(METHOD_DESCRIPTION_PARAMETER_NAME, AnnotationNode)

                if (methodToInvokeDescriptionAnnotation != null) {
                    rawMethodToInvokeDescription = MethodDescriptionReader.findMethodFromAnnotation(methodToInvokeDescriptionAnnotation)
                }
            }

            if (rawMethodToInvokeDescription == null) {
                def methodToInvokeName = context.targetMethod.name.removeCamelCasePrefixes(PREFIXES, true)
                rawMethodToInvokeDescription = new Method(methodToInvokeName, context.targetMethod.desc)
            }

            def mappedMethodToInvokeDescription = TransmutationRemapper.mapMethod(context.viewMethodMeta.view.targetType, rawMethodToInvokeDescription)

            context.result << new MethodInvokeStaticViewMethodMapper(mappedMethodToInvokeDescription)
        }
    }
}
